#!/usr/bin/env python

import sys

import collections
import getopt
import scipy
import math
import operator as op

from prefixURI import pp

import optimise
import svgball

def hexcolor(r,g,b):
    if isinstance(r,float) and r<=1 and g<=1 and b<=1:
        return "#%02x%02x%02x"%(255*r,255*g,255*b)
    else:
        return "#%02x%02x%02x"%(r,g,b)

def splitthousands(s, sep=','):
    if not isinstance(s, str): s=str(int(s))
    if len(s) <= 3: return s  
    return splitthousands(s[:-3], sep) + sep + s[-3:]

def help(): 
    sys.stderr.write("""
python ball.py -p <predicates> [-l <limit>] [-n <nodes>] [-s] [-o]

Options: 
  -p    a comma-separated list of predicates (i.e. columns), to include, f.x. "0,16"
  -l    only include nodes with more than <limit> edges
  -n    include only <nodes> biggest nodes.
  -s    do not scale nodes by size
  -o    do NOT optimse the node placement (i.e. clustering)
""")
    

args,files=getopt.getopt(sys.argv[1:],"sp:l:n:oh")
args=dict(args)

if "-h" in args: 
    help() 
    sys.exit(-1)


# all data files hardcoded :)
rows=[eval(x) for x in file("data/rows")]
cols=[x.strip() for x in file("data/cols")]

d=scipy.load("data/matrix.npy")

# Triple size
sizes=[x.strip().split() for x in file("data/hostsize")]
sizes=dict([(x[1], int(x[0])) for x in sizes])


preds=eval(args["-p"])

nodec=None
if "-n" in args: 
    nodec=int(args["-n"])

limit=0
if "-l" in args:
    assert not nodec
    limit=int(args["-l"])

SAMESIZE=False
if "-s" in args:
    SAMESIZE=True

nodes={}
for x in rows:
    if x[0] not in nodes:
        nodes[x[0]]=len(nodes)
    if x[1] not in nodes:
        nodes[x[1]]=len(nodes)

if nodec:
    counts=[x.sum() for x in d[:,preds]]
    counts.sort(reverse=True)
    #print counts[:60]
    limit=counts[nodec]
    if limit==0: limit=1
    sys.stderr.write("To have %d nodes we set limit=%d\n"%(nodec, limit))
    
    
    

gnodes=set()
edges={}

sum=0
count=0
for i,x in enumerate(d[:,preds]):
    sx=x.sum()
    if sx>=limit:
        gnodes.add(rows[i][0])
        gnodes.add(rows[i][1])

        nonzeropreds=filter(lambda a: x[a[0]]!=0, enumerate(preds))
        classes=["pred%d"%p[1] for p in nonzeropreds]
        classes=" "+" ".join(classes)

        #xs=[splitthousands(v) for v in x]
        attr=collections.defaultdict(int, zip(preds,x))

        k=tuple(sorted((nodes[rows[i][0]],nodes[rows[i][1]])))
        
        if k not in edges:
            edges[k]=[sx, classes, attr]
        else:
            edges[k][0]+=sx
            edges[k][1]+=classes
            for p, v in attr.items():
                edges[k][2][p]+=v
            

        count+=1
        sum+=sx

sys.stderr.write("%d nodes\n"%len(gnodes))
sys.stderr.write("%d/%d over the limit, average %.2f\n"%(count,d.shape[0],sum/float(count)))

ballnodes=[]
nodeindexmap={}

mins=min(sizes.values())
maxs=max(sizes.values())
lmaxs=math.log(maxs,5)


import pylab
cm=pylab.cm.RdYlBu_r

for n in gnodes:
    try: 
        #size=1+5.0*(sizes[n]-mins)/maxs
        
        size=1+2.0*math.log(sizes[n],2)/lmaxs

    except KeyError:
        size=1
        sizes[n]=1

    if SAMESIZE: size=1
    sys.stderr.write("%.2f\n"%size)
    #size*=0.5
    nodeindexmap[nodes[n]]=len(ballnodes)
    ci=int(255.0*math.log(sizes[n],5)/lmaxs)
    #sys.stderr.write("%d\n"%ci)
    c=cm(ci)[:3]
    
    ballnodes.append((n, size, hexcolor(*c)))


balledges=[]

mw=max([x[0] for x in edges.values()])
mw=math.sqrt(mw)
for e in edges.items():
    attr="count='%s'"%(" ".join(["%d:%d"%p for p in e[1][2].items()]))
    
    balledges.append((nodeindexmap[e[0][0]], nodeindexmap[e[0][1]], 10*math.sqrt(e[1][0])/mw, None, e[1][1], attr))

if "-o" not in args: 
    ballnodes,balledges=optimise.optimise(ballnodes, balledges, 20,5)
ball=svgball.SVGBall(1000,800)

ball.nodes(ballnodes, 12)
ball.edges(balledges)

ball.header()
ball.write()

x=800
y=50



print """<text class='theader' text-anchor='end' x='%d' y='%d'>count</text>"""%(x-20, y-25)
print """<text class='theader' x='%d' y='%d'>predicate</text>"""%(x, y-25)
print """<line class='theader' x1='%d' y1='%d' x2='%d' y2='%d' stroke-width='1' />"""%(x-75,y-18,x+100,y-18)

if type(preds)==int: preds=(preds,)

preds=[(p, d[:,p].sum(), pp(cols[p][1:-1])) for p in preds]

preds.sort(key=op.itemgetter(1))


for i,p in enumerate(preds):
    print """<text class='predcount' x='%d' y='%d'>%s</text>"""%(x-20, y+i*25,splitthousands(p[1]))
    print """<text class='predchooser' id='choose%d' font-weight='bold' x='%d' y='%d'>%s</text>"""%(p[0], x, y+i*25,p[2])

i+=1
print """<text class='instructions' text-anchor='start' x='%d' y='%d'>click to toggle predicate</text>"""%(x, i*25+y)


print """<!--This is ridicolous-->\n<text x='30' y='30' font-size='16' id='count' style='display: none' >
<tspan class='countline' >Between</tspan>
<tspan class='countline' dy='1.2em' id='count1'> </tspan>
<tspan class='countline' dy='1.2em'>&amp;</tspan>
<tspan class='countline' dy='1.2em' id='count2'> </tspan>
<tspan class='countline' dy='1.2em' id='count3'> </tspan>
</text>\n"""

print """"""
    
ball.footer()
