#!/usr/bin/env python 


import sys
import math

script="""
<script type="text/ecmascript" xlink:href="prototype.js" ></script>
<script type="text/ecmascript" xlink:href="ball.js" ></script>
"""


class SVGBall(object): 

    def __init__(self,w=800,h=800,irad_ratio=0.9):
        self.w=w
        self.h=h
        size=min(self.w, self.h)
        self.rad=size/3.0
        self.irad=irad_ratio*self.rad
        self.cx=size/2.0
        self.cy=size/2.0

        self.script=script

    
    def header(self, out=sys.stdout):
        out.write("""<?xml version="1.0" encoding="UTF-8" standalone="no"?>
        <?xml-stylesheet href="ball.css" type="text/css"?>
<svg
   xmlns:svg="http://www.w3.org/2000/svg"
   xmlns:xlink="http://www.w3.org/1999/xlink"
   xmlns="http://www.w3.org/2000/svg"
   version="1.0"
   width="%d"
   height="%d"
   onload="startup(evt)"
>

%s
<g>
"""%(self.w,self.h,self.script))
        out.write("""<circle cx="%d" cy="%d" r="%d" fill="none" stroke="brown" stroke-width="2" />\n"""%(self.cx,self.cy,self.irad))
    
    def nodes(self, nodes, fontsize=10):
        """Each node is
        (label, size, [color])
        """

        r=math.pi
        #inc=2*math.pi/len(nodes)

        total=sum([n[1] for n in nodes])
        inc=2*math.pi/total
        scaling=0.95*(inc/(2*math.pi))*(self.rad*math.pi)
        
        sys.stderr.write("%.2f, %.2f\n"%(inc,scaling))
        
        self.points={}
        self._nodes=[]

        radjust=fontsize/(2*self.irad)
        radjust=0

        i=0
        for i,s in enumerate(nodes):
            r+=inc*s[1]/2
            # each s is (label, size)
            rot=(360*r/(2*math.pi))%360

            col="blue"
            if len(s)>2: 
                col=s[2]
            
            trans="rotate(%f, %d, %d)"%(rot,self.cx,self.cy)
            #if rot>90 and rot<270:
            #    trans=trans+", translate(%(x)d, %(y)d), scale(-1,-1), translate(-%(x)d,-%(y)d) "%{"x": 4*len(s)+self.cx+self.rad, "y":self.cy}
            
            self.points[i]=(self.cx+self.irad*math.cos(r-radjust), 
                            self.cy+self.irad*math.sin(r-radjust))

            self._nodes.append({"fs":fontsize, "i":i,
                                "tx":self.cx+self.rad, "ty":self.cy,
                                "trans":trans, "txt":s[0],
                                "cx": self.points[i][0], "cy": self.points[i][1],
                                "rad": scaling*s[1], "col":col })

            r+=inc*s[1]/2

        
        return self.points

    def edges(self, edges, dcolor="grey", dwidth=1, r=0.8):
        """Each edge is:
        (node1, node2, [width], [color], [classes])
        color is html-hex """
        self._edges=[]
        for e in edges: 
            color=dcolor
            width=dwidth
            classes=""
            attr=""
            if len(e)>2:
                width=e[2]
            if len(e)>3 and e[3]!=None: 
                color=e[3]
            if len(e)>4:
                classes=e[4]
            if len(e)>5:
                attr=e[5]


            p1=self.points[e[0]]
            p2=self.points[e[1]]
            
            c1=((p1[0]-self.cx)*r+self.cx,
                (p1[1]-self.cy)*r+self.cy)
            c2=((p2[0]-self.cx)*r+self.cx,
                (p2[1]-self.cy)*r+self.cy)

            # c3=((p1[0]-self.cx)*r*1.1+self.cx,
            #     (p1[1]-self.cy)*r*1.1+self.cy)
            # c4=((p2[0]-self.cx)*r*1.1+self.cx,
            #     (p2[1]-self.cy)*r*1.1+self.cy)

            
            self._edges.append({"n1":e[0], "n2":e[1],
                                "class":classes, "attr": attr,
                                "x1":p1[0], "y1":p1[1],
                                "cx1":c1[0], "cy1":c1[1],
                                "cx2":c2[0], "cy2":c2[1],
                                "x2":p2[0], "y2":p2[1],
                                "col":color, "w":width})

    def write(self, out=sys.stdout):
        self.writeedges(out)
        self.writenodes(out)

    def writeedges(self, out=sys.stdout):
        for e in self._edges:
            out.write("""<path class='edge node%(n1)d node%(n2)d%(class)s' d='M%(x1)d,%(y1)d C%(cx1)d,%(cy1)d %(cx2)d,%(cy2)d %(x2)d,%(y2)d' style='opacity: 0.5' fill='none' stroke='%(col)s' orig-stroke-width='%(w)f' stroke-width='%(w)f' %(attr)s/>\n"""%e)

    def writenodes(self, out=sys.stdout):
        for n in self._nodes:
            out.write("""<text class='label' style='font-size: %(fs)dpx; ' font-weight='normal' fill='black' id='text%(i)d' x='%(tx)d' y='%(ty)d' transform='%(trans)s'>%(txt)s</text>\n"""%n)
            out.write("""<circle class='circle' id='circ%(i)d' cx='%(cx)f' cy='%(cy)f' r='%(rad)d' fill='%(col)s' stroke='none' />\n"""%n)
            

    def footer(self, out=sys.stdout): 
        out.write("</g></svg>\n")



if __name__=='__main__':
    import fileinput
    import random

    strings = [x.strip() for x in fileinput.input()]
    
    edges=[]
    for s in range(len(strings)):
        for s2 in range(len(strings)): 
            if s==s2: continue
            if random.random()>0.01: continue
            edges.append((s,s2))

    ball=SVGBall(800,800)
    ball.header()
    ball.nodes(strings)
    ball.edges(edges)
    ball.footer()
